/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DTS_MARVELL_MMP2_POWER_H
#define __DTS_MARVELL_MMP2_POWER_H

#define MMP2_POWER_DOMAIN_GPU		0
#define MMP2_POWER_DOMAIN_AUDIO		1
#define MMP3_POWER_DOMAIN_CAMERA	2

#define MMP2_NR_POWER_DOMAINS		3

#endif
