/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */

#ifndef __DT_BINDINGS_QCOM_GPR_H
#define __DT_BINDINGS_QCOM_GPR_H

/* DOMAINS */

#define GPR_DOMAIN_ID_MODEM	1
#define GPR_DOMAIN_ID_ADSP	2
#define GPR_DOMAIN_ID_APPS	3

/* Static Services */

#define GPR_APM_MODULE_IID		1
#define GPR_PRM_MODULE_IID		2
#define GPR_AMDB_MODULE_IID		3
#define GPR_VCPM_MODULE_IID		4

#endif /* __DT_BINDINGS_QCOM_GPR_H */
