/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */

#ifndef _UAPI_LINUX_PIDFD_H
#define _UAPI_LINUX_PIDFD_H

#include <linux/types.h>
#include <linux/fcntl.h>

/* Flags for pidfd_open().  */
#define PIDFD_NONBLOCK O_NONBLOCK

#endif /* _UAPI_LINUX_PIDFD_H */
