/* SPDX-License-Identifier: (GPL-2.0 or MIT) */
/*
 * Copyright (C) 2021 Renesas Electronics Corp.
 */
#ifndef __DT_BINDINGS_POWER_R8A779F0_SYSC_H__
#define __DT_BINDINGS_POWER_R8A779F0_SYSC_H__

/*
 * These power domain indices match the Power Domain Register Numbers (PDR)
 */

#define R8A779F0_PD_A1E0D0C0		0
#define R8A779F0_PD_A1E0D0C1		1
#define R8A779F0_PD_A1E0D1C0		2
#define R8A779F0_PD_A1E0D1C1		3
#define R8A779F0_PD_A1E1D0C0		4
#define R8A779F0_PD_A1E1D0C1		5
#define R8A779F0_PD_A1E1D1C0		6
#define R8A779F0_PD_A1E1D1C1		7
#define R8A779F0_PD_A2E0D0		16
#define R8A779F0_PD_A2E0D1		17
#define R8A779F0_PD_A2E1D0		18
#define R8A779F0_PD_A2E1D1		19
#define R8A779F0_PD_A3E0		20
#define R8A779F0_PD_A3E1		21

/* Always-on power area */
#define R8A779F0_PD_ALWAYS_ON		64

#endif /* __DT_BINDINGS_POWER_R8A779A0_SYSC_H__*/
